@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/builtin.c,v 1.16 2004/02/27 03:50:15 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/builtin.c,v 1.17 2004/04/11 06:06:51 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -172,7 +172,7 @@ BuiltinType (char *format, Type **type)
     if (ref)
 	t = NewTypeRef (t, False);
     if (array)
-	t = NewTypeArray (t, dims);
+	t = NewTypeArray (t, dims, False);
     if (hash)
 	t = NewTypeHash (t, k);
     *type = t;
