@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/file.c,v 1.59 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/file.c,v 1.61 2004/04/16 05:26:45 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -778,7 +778,7 @@ FileFilter (char *program, char *args[], Value filev, int *errp)
 
     /* set up process files */
     for (i = 0; i < 3; i++) {
-	Value f = BoxValue (filev->array.values, i);
+	Value f = ArrayValue (&filev->array, i);
 	if (i == 0 && !(f->file.flags & FileReadable)) {
 	    RaiseStandardException (exception_invalid_argument,
 				    "File::filter: process input not readable",
@@ -835,7 +835,7 @@ FileFilter (char *program, char *args[], Value filev, int *errp)
 	RETURN(0);
     }
     for (i = 0; i < 3; i++) {
-	Value f = BoxValue (filev->array.values, i);
+	Value f = ArrayValue (&filev->array, i);
 	if (f->file.flags & FilePipe)
 	    f->file.pid = pid;
     }
@@ -859,10 +859,10 @@ FileMakePipe (int *errp)
     files = NewArray (False, False, typePrim[rep_file], 1, &two);
     file = FileCreate (fds[0], FileReadable);
     file->file.flags |= FilePipe;
-    BoxValueSet (files->array.values, 0, file);
+    ArrayValueSet (&files->array, 0, file);
     file = FileCreate (fds[1], FileWritable);
     file->file.flags |= FilePipe;
-    BoxValueSet (files->array.values, 1, file);
+    ArrayValueSet (&files->array, 1, file);
     RETURN (files);
 }
 
@@ -1513,12 +1513,14 @@ FilePutArgType (Value f, ArgType *at)
 }
 
 static void
-FilePutDimensions (Value f, ExprPtr dims)
+FilePutDimensions (Value f, ExprPtr dims, Bool resizable)
 {
     while (dims)
     {
 	if (dims->tree.left)
 	    PrettyExpr (f, dims->tree.left, -1, 0, False, 0);
+	else if (resizable)
+	    FilePuts (f, "...");
 	else
 	    FilePuts (f, "*");
 	if (dims->tree.right)
@@ -1574,7 +1576,7 @@ FilePutSubscriptType (Value f, Type *t, Bool minimal)
 	break;
     case type_array:
 	FilePuts (f, "[");
-	FilePutDimensions (f, t->array.dimensions);
+	FilePutDimensions (f, t->array.dimensions, t->array.resizable);
 	FilePuts (f, "]");
 	FilePutSubscriptType (f, t->array.type, minimal);
 	break;
