@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/nickle.h,v 1.113 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/nickle.h,v 1.114 2004/04/11 06:06:52 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -396,7 +396,8 @@ typedef struct _instStruct {
 
 typedef struct _instArray {
     InstBase	inst;
-    int		ndim;
+    short    	ndim;
+    short	resizable;
     TypePtr	type;
 } InstArray;
 
