@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/pretty.c,v 1.65 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/pretty.c,v 1.66 2004/04/11 06:06:52 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -146,7 +146,7 @@ PrettyParameters (Value f, Expr *e, Bool nest, ProfileData *pd)
 {
     while (e) 
     {
-	if (e->tree.left->base.tag == DOTS)
+	if (e->tree.left->base.tag == DOTDOTDOT)
 	{
 	    PrettyExpr (f, e->tree.left->tree.left, -1, 0, nest, pd);
 	    FilePuts (f, "...");
@@ -174,7 +174,7 @@ PrettyArrayInits (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
 	e = e->tree.right;
 	if (e)
 	{
-	    if (e->tree.left->base.tag == DOTS)
+	    if (e->tree.left->base.tag == DOTDOTDOT)
 		FilePuts (f, " ");
 	    else
 		FilePuts (f, ", ");
@@ -191,7 +191,7 @@ PrettyArrayInit (Value f, Expr *e, int level, Bool nest, ProfileData *pd)
 	PrettyArrayInits (f, e->tree.left, level, nest, pd);
 	FilePuts (f, " }");
 	break;
-    case DOTS:
+    case DOTDOTDOT:
 	FilePuts (f, "...");
 	break;
     default:
