@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/type.c,v 1.59 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/type.c,v 1.62 2004/04/16 05:26:45 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -179,7 +179,7 @@ NewTypeFunc (Type *ret, ArgType *args)
 }
 
 Type *
-NewTypeArray (Type *type, Expr *dimensions)
+NewTypeArray (Type *type, Expr *dimensions, Bool resizable)
 {
     ENTER ();
     Type   *t;
@@ -189,6 +189,7 @@ NewTypeArray (Type *type, Expr *dimensions)
     t->array.type = type;
     t->array.dimensions = dimensions;
     t->array.storage = DimStorageNone;
+    t->array.resizable = resizable;
     RETURN (t);
 }
 
@@ -600,7 +601,7 @@ static Type *
 TypeUnaryRef (Type *ref)
 {
     if (TypePoly (ref))
-	ref = typePoly;
+	return typePoly;
     if (ref->base.tag == type_ref)
 	return ref->ref.ref;
     return 0;
@@ -1098,15 +1099,16 @@ TypeCompatibleAssign (TypePtr a, Value b)
 		if (TypePoly (ArrayType(&b->array)))
 		{
 		    int	i;
-		    BoxPtr  box = b->array.values;
 
-		    for (i = 0; i < box->nvalues; i++)
-			if (BoxValueGet (box, i) &&
-			    !TypeCompatibleAssign (a->array.type,
-						   BoxValueGet (box, i)))
+		    for (i = 0; i < ArrayNvalues(&b->array); i++)
+		    {
+			Value	v = ArrayValueGet (&b->array, i);
+			if (v &&
+			    !TypeCompatibleAssign (a->array.type, v))
 			{
 			    return False;
 			}
+		    }
 		    return True;
 		}
 		else
@@ -1203,7 +1205,7 @@ TypeInit (void)
 					  NewTypeElt (typePrim[rep_float], 0)));
     MemAddRoot (typeField);
     
-    typeArrayInt = NewTypeArray (typePrim[rep_integer], 0);
+    typeArrayInt = NewTypeArray (typePrim[rep_integer], 0, False);
     MemAddRoot (typeArrayInt);
 
     TypeCheckStack = StackCreate ();
