@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/value.h,v 1.97 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/value.h,v 1.101 2004/04/16 05:26:45 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -71,6 +71,12 @@ AtomListPtr  NewAtomList (AtomListPtr next, Atom atom);
 #if HAVE_STDINT_H
 
 # include <stdint.h>
+
+#define PtrToInt(p)	((int) (intptr_t) (p))
+#define PtrToUInt(p)	((unsigned) (uintptr_t) (p))
+#define IntToPtr(i)	((void *) (intptr_t) (i))
+#define UIntToPtr(u)	((void *) (uintptr_t) (u))
+
 # if HAVE_UINT64_T
 
 /*
@@ -94,6 +100,11 @@ typedef int32_t	    signed_digit;
 
 #else
 
+#define PtrToInt(p)	((int) (p))
+#define PtrToUInt(p)	((unsigned) (p))
+#define IntToPtr(i)	((void *) (i))
+#define UIntToPtr(u)	((void *) (u))
+
 # if SIZEOF_UNSIGNED_LONG_LONG == 8 || SIZEOF_UNSIGNED_LONG == 8
 #  define DIGITBITS 32
 # else
@@ -289,7 +300,7 @@ extern ValueRep    RefRep, StructRep, UnionRep, HashRep;
 extern ValueRep	   FuncRep, ThreadRep;
 extern ValueRep    SemaphoreRep, ContinuationRep, UnitRep, BoolRep;
 
-#define NewInt(i)	((Value) (((i) << 1) | 1))
+#define NewInt(i)	((Value) IntToPtr ((((i) << 1) | 1)))
 #define IntSign(i)	((i) < 0 ? Negative : Positive)
 
 #define NICKLE_INT_BITS	    ((sizeof (int) * 8) - 1)
@@ -301,10 +312,10 @@ extern ValueRep    SemaphoreRep, ContinuationRep, UnitRep, BoolRep;
 #define One NewInt(1)
 #define Zero NewInt(0)
 
-#define ValueIsPtr(v)	((((int) (v)) & 1) == 0)
-#define ValueIsInt(v)	((((int) (v)) & 1) != 0)
-#define ValueInt(v)	(((int) (v)) >> 1)
-#define ValueUInt(v)	(((unsigned int) (v)) >> 1)
+#define ValueIsPtr(v)	((PtrToInt(v) & 1) == 0)
+#define ValueIsInt(v)	(!ValueIsPtr(v))
+#define ValueInt(v)	(PtrToInt (v) >> 1)
+#define ValueUInt(v)	(PtrToUInt (v) >> 1)
 
 #define ValueRep(v) (ValueIsInt(v) ? &IntRep : (v)->value.type)
 #define ValueIsInteger(v) (ValueRep(v) == &IntegerRep)
@@ -384,6 +395,7 @@ typedef struct _typeArray {
     ExprPtr	dimensions;
     int		dims;
     DimStorage	storage;
+    Bool	resizable;
     union {
 	BoxPtr	global;
 	struct {
@@ -451,7 +463,7 @@ Type	*NewTypeName (ExprPtr expr, SymbolPtr name);
 Type	*NewTypeRef (Type *ref, Bool pointer);
 Type	*NewTypePointer (Type *ref);
 Type	*NewTypeFunc (Type *ret, ArgType *args);
-Type	*NewTypeArray (Type *type, ExprPtr dimensions);
+Type	*NewTypeArray (Type *type, ExprPtr dimensions, Bool resizable);
 Type	*NewTypeHash (Type *type, Type *keyType);
 Type	*NewTypeStruct (StructTypePtr structs);
 Type	*NewTypeUnion (StructTypePtr structs, Bool enumeration);
@@ -544,16 +556,40 @@ typedef struct _string {
 
 #define StringChars(s)	    ((char *) ((s) + 1))
 
+/*
+ * Resizable arrays are actually vectors of single entry
+ * boxes.  Otherwise shrinking the array leaves old references
+ * dangling.
+ */
+
+typedef struct _boxVector {
+    DataType	*data;
+    int		nvalues;
+    TypePtr	type;
+} BoxVector, *BoxVectorPtr;
+
+#define BoxVectorBoxes(v)   ((BoxPtr *) ((v) + 1))
+
 typedef struct _array {
     BaseValue	base;
     unsigned int	resizable : 1;
     unsigned int	ndim : (sizeof (int) * 8 - 1);
-    BoxPtr	values;
+    union {
+	BoxPtr		fix;
+	BoxVectorPtr	resize;
+    } u;
 } Array;
 
 #define ArrayDims(a)	    ((int *) ((a) + 1))
 #define ArrayLimits(a)	    (ArrayDims(a) + (a)->ndim)
-#define ArrayType(a)	    ((a)->values->u.type)
+#define ArrayConstant
+#define ArrayNvalues(a)	    ((a)->resizable ? (a)->u.resize->nvalues : (a)->u.fix->nvalues)
+#define ArrayValueBox(a,i)  ((a)->resizable ? BoxVectorBoxes((a)->u.resize)[i] : (a)->u.fix)
+#define ArrayValueElt(a,i)  ((a)->resizable ? 0 : (i))
+#define ArrayType(a)	    ((a)->resizable ? (a)->u.resize->type : (a)->u.fix->u.type)
+#define ArrayValue(a,i)	    (BoxValue(ArrayValueBox(a,i),ArrayValueElt(a,i)))
+#define ArrayValueGet(a,i)  (BoxValueGet(ArrayValueBox(a,i),ArrayValueElt(a,i)))
+#define ArrayValueSet(a,i,v) (BoxValueSet(ArrayValueBox(a,i),ArrayValueElt(a,i), v))
 
 typedef struct _io_chain {
     DataType		*data;
@@ -785,6 +821,7 @@ typedef struct _hashTable {
     TypePtr	type;
     TypePtr	keyType;
     BoxPtr	elts;
+    Value	def;
 } HashTable, *HashTablePtr;
 
 typedef union _value {
@@ -910,6 +947,7 @@ void	ArraySetDimensions (Value av, int *dims);
 Value	NewHash (Bool constant, TypePtr keyType, TypePtr valueType);
 Value	HashGet (Value hv, Value key);
 void	HashSet (Value hv, Value key, Value value);
+void	HashSetDef (Value hv, Value def);
 Value	HashKeys (Value hv);
 Value	HashRef (Value hv, Value key);
 Value	HashTest (Value hv, Value key);
@@ -929,7 +967,7 @@ unsigned    StringGet (char *src, int i);
 static inline Value
 NewRef (BoxPtr box, int element)
 {
-    int	    c = ((unsigned) (&BoxElements(box)[element])) % REF_CACHE_SIZE;
+    int	    c = (PtrToUInt (&BoxElements(box)[element])) % REF_CACHE_SIZE;
     Value   *re = (Value *) (DataCacheValues(refCache)) + c;
     Value   ret = *re;
 
